<?php

namespace Survey\Gang\Policies;

use Litepie\User\Contracts\UserPolicy;
use Survey\Gang\Models\Gang;

class GangPolicy
{

    /**
     * Determine if the given user can view the gang.
     *
     * @param UserPolicy $user
     * @param Gang $gang
     *
     * @return bool
     */
    public function view(UserPolicy $user, Gang $gang)
    {
        if ($user->canDo('gang.gang.view') && $user->isAdmin()) {
            return true;
        }

        return $gang->user_id == user_id() && $gang->user_type == user_type();
    }

    /**
     * Determine if the given user can create a gang.
     *
     * @param UserPolicy $user
     * @param Gang $gang
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('gang.gang.create');
    }

    /**
     * Determine if the given user can update the given gang.
     *
     * @param UserPolicy $user
     * @param Gang $gang
     *
     * @return bool
     */
    public function update(UserPolicy $user, Gang $gang)
    {
        if ($user->canDo('gang.gang.edit') && $user->isAdmin()) {
            return true;
        }

        return $gang->user_id == user_id() && $gang->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given gang.
     *
     * @param UserPolicy $user
     * @param Gang $gang
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Gang $gang)
    {
        return $gang->user_id == user_id() && $gang->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given gang.
     *
     * @param UserPolicy $user
     * @param Gang $gang
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Gang $gang)
    {
        if ($user->canDo('gang.gang.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given gang.
     *
     * @param UserPolicy $user
     * @param Gang $gang
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Gang $gang)
    {
        if ($user->canDo('gang.gang.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
